---
# .. vim: foldmarker=[[[,]]]:foldmethod=marker

{% set section_heading = ansible_full_role_name + " default variables [[[" %}
# {{ section_heading }}
# {{ '=' * (section_heading|length) }}

# .. contents:: Sections
#    :local:
#
# .. include:: includes/all.rst
